@@ -586,15 +586,18 @@ gnutls_pkcs11_init (unsigned int flags, const char *deprecated_config_file)
  **/
 int gnutls_pkcs11_reinit (void)
 {
+  unsigned i;
   int rv;
 
-  rv = p11_kit_initialize_registered ();
-  if (rv != CKR_OK)
+  for (i = 0; i < active_providers; i++)
     {
-      gnutls_assert ();
-      _gnutls_debug_log ("Cannot initialize registered module: %s\n",
-                         p11_kit_strerror (rv));
-      return GNUTLS_E_INTERNAL_ERROR;
+      if (providers[i].module != NULL) 
+        {
+          rv = p11_kit_initialize_module(providers[i].module);
+          if (rv != CKR_OK)
+            _gnutls_debug_log ("Cannot initialize registered module '%s': %s\n", 
+              providers[i].info.library_description, p11_kit_strerror (rv));
+        }
     }
 
   return 0;
@@ -625,6 +628,7 @@ gnutls_pkcs11_deinit (void)
     {
       if (providers[i].initialized)
         p11_kit_finalize_module (providers[i].module);
+      gnutls_free (providers[i].slots);
     }
   active_providers = 0;
   
@@ -634,6 +638,8 @@ gnutls_pkcs11_deinit (void)
   
   gnutls_pkcs11_set_pin_function (NULL, NULL);
   gnutls_pkcs11_set_token_function (NULL, NULL);
+  p11_kit_pin_unregister_callback (P11_KIT_PIN_FALLBACK, p11_kit_pin_file_callback,
+                                   NULL);
 }
 
 /**
